/* This file is generated automatically.
 * Source file: ../raddb/dictionary
 *
 * Naming convention:
 *     DA_XXX          -   Dictionary Attribute XXX
 *     DV_XXX_YYY      -   Dictionary Value YYY corresponding to attribute XXX
 */
/* 
 *  $Id: raddict.h,v 1.5 2000/12/18 17:55:10 gray Exp $
 * 
 */
#define DA_USER_NAME 1
#define DA_PASSWORD 2
#define DA_CHAP_PASSWORD 3
#define DA_NAS_IP_ADDRESS 4
#define DA_NAS_PORT_ID 5
#define DA_SERVICE_TYPE 6
#define DA_FRAMED_PROTOCOL 7
#define DA_FRAMED_IP_ADDRESS 8
#define DA_FRAMED_IP_NETMASK 9
#define DA_FRAMED_ROUTING 10
#define DA_FILTER_ID 11
#define DA_FRAMED_MTU 12
#define DA_FRAMED_COMPRESSION 13
#define DA_LOGIN_IP_HOST 14
#define DA_LOGIN_SERVICE 15
#define DA_LOGIN_TCP_PORT 16
#define DA_REPLY_MESSAGE 18
#define DA_CALLBACK_NUMBER 19
#define DA_CALLBACK_ID 20
#define DA_FRAMED_ROUTE 22
#define DA_FRAMED_IPX_NETWORK 23
#define DA_STATE 24
#define DA_CLASS 25
#define DA_VENDOR_SPECIFIC 26
#define DA_SESSION_TIMEOUT 27
#define DA_IDLE_TIMEOUT 28
#define DA_TERMINATION_ACTION 29
#define DA_CALLED_STATION_ID 30
#define DA_CALLING_STATION_ID 31
#define DA_NAS_IDENTIFIER 32
#define DA_PROXY_STATE 33
#define DA_LOGIN_LAT_SERVICE 34
#define DA_LOGIN_LAT_NODE 35
#define DA_LOGIN_LAT_GROUP 36
#define DA_FRAMED_APPLETALK_LINK 37
#define DA_FRAMED_APPLETALK_NETWORK 38
#define DA_FRAMED_APPLETALK_ZONE 39
#define DA_ACCT_STATUS_TYPE 40
#define DA_ACCT_DELAY_TIME 41
#define DA_ACCT_INPUT_OCTETS 42
#define DA_ACCT_OUTPUT_OCTETS 43
#define DA_ACCT_SESSION_ID 44
#define DA_ACCT_AUTHENTIC 45
#define DA_ACCT_SESSION_TIME 46
#define DA_ACCT_INPUT_PACKETS 47
#define DA_ACCT_OUTPUT_PACKETS 48
#define DA_ACCT_TERMINATE_CAUSE 49
#define DA_ACCT_MULTI_SESSION_ID 50
#define DA_ACCT_LINK_COUNT 51
#define DA_CHAP_CHALLENGE 60
#define DA_NAS_PORT_TYPE 61
#define DA_PORT_LIMIT 62
#define DA_LOGIN_LAT_PORT 63
#define DA_CONNECT_INFO 77
#define DA_USER_UID 99
/* 
 *  Cistron-Radius compatibility
 * 
 */
#define DA_HUNTGROUP_NAME 221
#define DA_USER_CATEGORY 1029
#define DA_GROUP_NAME 1030
#define DA_SIMULTANEOUS_USE 1034
#define DA_STRIP_USER_NAME 1035
#define DA_FALL_THROUGH 1036
#define DA_ADD_PORT_TO_IP_ADDRESS 1037
#define DA_EXEC_PROGRAM 1038
#define DA_EXEC_PROGRAM_WAIT 1039
#define DA_HINT 1040
#define DA_PAM_AUTH 1041
#define DA_LOGIN_TIME 1042
/*  Attributes used by GNU Radius
 */
#define DA_REPLACE_USER_NAME 2001
#define DA_REWRITE_FUNCTION 2002
/*  These are used by rewriting functions:
 */
#define DA_ORIG_USER_NAME 2101
#define DA_ORIG_NAS_PORT_ID 2105
#define DA_ORIG_ACCT_SESSION_ID 2144
/* 
 *  Non-Protocol Attributes
 *  These attributes are used internally by the server
 * 
 */
#define DA_EXPIRATION 21
#define DA_AUTH_TYPE 1000
#define DA_MENU 1001
#define DA_TERMINATION_MENU 1002
#define DA_PREFIX 1003
#define DA_SUFFIX 1004
#define DA_GROUP 1005
#define DA_CRYPT_PASSWORD 1006
#define DA_CONNECT_RATE 1007
/* 
 *  Integer Translations
 * 
 */
/*  User Types
 */
#define DV_SERVICE_TYPE_LOGIN_USER 1
#define DV_SERVICE_TYPE_FRAMED_USER 2
#define DV_SERVICE_TYPE_CALLBACK_LOGIN_USER 3
#define DV_SERVICE_TYPE_CALLBACK_FRAMED_USER 4
#define DV_SERVICE_TYPE_OUTBOUND_USER 5
#define DV_SERVICE_TYPE_ADMINISTRATIVE_USER 6
#define DV_SERVICE_TYPE_NAS_PROMPT_USER 7
#define DV_SERVICE_TYPE_AUTHENTICATE_ONLY 8
#define DV_SERVICE_TYPE_CALL_CHECK 10
/*  GNU Radius Extension
 */
#define DV_SERVICE_TYPE_RADIUS_ADMINISTRATIVE_USER 20
/*  Framed Protocols
 */
#define DV_FRAMED_PROTOCOL_PPP 1
#define DV_FRAMED_PROTOCOL_SLIP 2
/*  Framed Routing Values
 */
#define DV_FRAMED_ROUTING_NONE 0
#define DV_FRAMED_ROUTING_BROADCAST 1
#define DV_FRAMED_ROUTING_LISTEN 2
#define DV_FRAMED_ROUTING_BROADCAST_LISTEN 3
/*  Framed Compression Types
 */
#define DV_FRAMED_COMPRESSION_NONE 0
#define DV_FRAMED_COMPRESSION_VAN_JACOBSON_TCP_IP 1
/*  Login Services
 */
#define DV_LOGIN_SERVICE_TELNET 0
#define DV_LOGIN_SERVICE_RLOGIN 1
#define DV_LOGIN_SERVICE_TCP_CLEAR 2
#define DV_LOGIN_SERVICE_PORTMASTER 3
/*  Status Types
 */
#define DV_ACCT_STATUS_TYPE_START 1
#define DV_ACCT_STATUS_TYPE_STOP 2
#define DV_ACCT_STATUS_TYPE_ALIVE 3
#define DV_ACCT_STATUS_TYPE_ACCOUNTING_ON 7
#define DV_ACCT_STATUS_TYPE_ACCOUNTING_OFF 8
/*  Authentication Types
 */
#define DV_ACCT_AUTHENTIC_RADIUS 1
#define DV_ACCT_AUTHENTIC_LOCAL 2
/*  Termination Options
 */
#define DV_TERMINATION_ACTION_DEFAULT 0
#define DV_TERMINATION_ACTION_RADIUS_REQUEST 1
/*  NAS Port Types, available in 3.3.1 and later
 */
#define DV_NAS_PORT_TYPE_ASYNC 0
#define DV_NAS_PORT_TYPE_SYNC 1
#define DV_NAS_PORT_TYPE_ISDN 2
#define DV_NAS_PORT_TYPE_ISDN_V120 3
#define DV_NAS_PORT_TYPE_ISDN_V110 4
/*  Acct Terminate Causes, available in 3.3.2 and later
 */
#define DV_ACCT_TERMINATE_CAUSE_USER_REQUEST 1
#define DV_ACCT_TERMINATE_CAUSE_LOST_CARRIER 2
#define DV_ACCT_TERMINATE_CAUSE_LOST_SERVICE 3
#define DV_ACCT_TERMINATE_CAUSE_IDLE_TIMEOUT 4
#define DV_ACCT_TERMINATE_CAUSE_SESSION_TIMEOUT 5
#define DV_ACCT_TERMINATE_CAUSE_ADMIN_RESET 6
#define DV_ACCT_TERMINATE_CAUSE_ADMIN_REBOOT 7
#define DV_ACCT_TERMINATE_CAUSE_PORT_ERROR 8
#define DV_ACCT_TERMINATE_CAUSE_NAS_ERROR 9
#define DV_ACCT_TERMINATE_CAUSE_NAS_REQUEST 10
#define DV_ACCT_TERMINATE_CAUSE_NAS_REBOOT 11
#define DV_ACCT_TERMINATE_CAUSE_PORT_UNNEEDED 12
#define DV_ACCT_TERMINATE_CAUSE_PORT_PREEMPTED 13
#define DV_ACCT_TERMINATE_CAUSE_PORT_SUSPENDED 14
#define DV_ACCT_TERMINATE_CAUSE_SERVICE_UNAVAILABLE 15
#define DV_ACCT_TERMINATE_CAUSE_CALLBACK 16
#define DV_ACCT_TERMINATE_CAUSE_USER_ERROR 17
#define DV_ACCT_TERMINATE_CAUSE_HOST_REQUEST 18
/* 
 *  Auth-Type translations
 * 
 */
#define DV_AUTH_TYPE_LOCAL 0
#define DV_AUTH_TYPE_SYSTEM 1
#define DV_AUTH_TYPE_SECURID 2
#define DV_AUTH_TYPE_CRYPT_LOCAL 3
#define DV_AUTH_TYPE_REJECT 4
#define DV_AUTH_TYPE_ACTIVCARD 4
/* 
 *  Extensions introduced by Cistron
 * 
 */
#define DV_AUTH_TYPE_MYSQL 252
#define DV_AUTH_TYPE_PAM 253
#define DV_AUTH_TYPE_ACCEPT 254
#define DV_FALL_THROUGH_NO 0
#define DV_FALL_THROUGH_YES 1
#define DV_ADD_PORT_TO_IP_ADDRESS_NO 0
#define DV_ADD_PORT_TO_IP_ADDRESS_YES 1
/* 
 * 	Configuration Values
 * 	comment out these two lines to turn account expiration off
 * 
 */
#define DV_SERVER_CONFIG_PASSWORD_EXPIRATION 30
#define DV_SERVER_CONFIG_PASSWORD_WARNING 5
