/* This file is part of GNU RADIUS.
 * Copyright (C) 2000, Sergey Poznyakoff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

static char rcsid[] = 
"$Id: strncasecmp.c,v 1.2 2000/12/07 15:32:25 gray Exp $";

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#if !defined(HAVE_STRNCASECMP)

int
strncasecmp(a, b, n)
	char *a;
	char *b;
	int   n;
{
	register int d;

	while (n--) {
		if (!*a)
			return -1;
		if (!*b)
			return 1;
		if ((d = tolower(a) - tolower(b)) != 0)
			return d;
	}
	return 0;		
}
		
#endif /* HAVE_STRNCASECMP */

