/* $Id: radmib.h,v 1.1 2000/04/26 10:38:13 gray Exp $ */
/* This file is automatically generated
 * Please, do not edit.
 */
#define MIB_radiusMIB 1,3,6,1,2,1,67
#define LEN_radiusMIB 7
#define MIB_KEY_radiusMIB 67
#define MIB_radiusAuthentication 1,3,6,1,2,1,67,1
#define LEN_radiusAuthentication 8
#define MIB_KEY_radiusAuthentication 1
#define MIB_radiusAuthServMIB 1,3,6,1,2,1,67,1,1
#define LEN_radiusAuthServMIB 9
#define MIB_KEY_radiusAuthServMIB 1
#define MIB_radiusAuthServMIBObjects 1,3,6,1,2,1,67,1,1,1
#define LEN_radiusAuthServMIBObjects 10
#define MIB_KEY_radiusAuthServMIBObjects 1
#define MIB_POS_radiusAuthServ 11
#define MIB_radiusAuthServ 1,3,6,1,2,1,67,1,1,1,1
#define LEN_radiusAuthServ 11
#define MIB_KEY_radiusAuthServ 1
#define MIB_radiusAuthServIdent 1,3,6,1,2,1,67,1,1,1,1,1
#define LEN_radiusAuthServIdent 12
#define MIB_KEY_radiusAuthServIdent 1
#define MIB_radiusAuthServUpTime 1,3,6,1,2,1,67,1,1,1,1,2
#define LEN_radiusAuthServUpTime 12
#define MIB_KEY_radiusAuthServUpTime 2
#define MIB_radiusAuthServResetTime 1,3,6,1,2,1,67,1,1,1,1,3
#define LEN_radiusAuthServResetTime 12
#define MIB_KEY_radiusAuthServResetTime 3
#define MIB_radiusAuthServConfigReset 1,3,6,1,2,1,67,1,1,1,1,4
#define LEN_radiusAuthServConfigReset 12
#define MIB_KEY_radiusAuthServConfigReset 4
#define MIB_radiusAuthServTotalAccessRequests 1,3,6,1,2,1,67,1,1,1,1,5
#define LEN_radiusAuthServTotalAccessRequests 12
#define MIB_KEY_radiusAuthServTotalAccessRequests 5
#define MIB_radiusAuthServTotalInvalidRequests 1,3,6,1,2,1,67,1,1,1,1,6
#define LEN_radiusAuthServTotalInvalidRequests 12
#define MIB_KEY_radiusAuthServTotalInvalidRequests 6
#define MIB_radiusAuthServTotalDupAccessRequests 1,3,6,1,2,1,67,1,1,1,1,7
#define LEN_radiusAuthServTotalDupAccessRequests 12
#define MIB_KEY_radiusAuthServTotalDupAccessRequests 7
#define MIB_radiusAuthServTotalAccessAccepts 1,3,6,1,2,1,67,1,1,1,1,8
#define LEN_radiusAuthServTotalAccessAccepts 12
#define MIB_KEY_radiusAuthServTotalAccessAccepts 8
#define MIB_radiusAuthServTotalAccessRejects 1,3,6,1,2,1,67,1,1,1,1,9
#define LEN_radiusAuthServTotalAccessRejects 12
#define MIB_KEY_radiusAuthServTotalAccessRejects 9
#define MIB_radiusAuthServTotalAccessChallenges 1,3,6,1,2,1,67,1,1,1,1,10
#define LEN_radiusAuthServTotalAccessChallenges 12
#define MIB_KEY_radiusAuthServTotalAccessChallenges 10
#define MIB_radiusAuthServTotalMalformedAccessRequests 1,3,6,1,2,1,67,1,1,1,1,11
#define LEN_radiusAuthServTotalMalformedAccessRequests 12
#define MIB_KEY_radiusAuthServTotalMalformedAccessRequests 11
#define MIB_radiusAuthServTotalBadAuthenticators 1,3,6,1,2,1,67,1,1,1,1,12
#define LEN_radiusAuthServTotalBadAuthenticators 12
#define MIB_KEY_radiusAuthServTotalBadAuthenticators 12
#define MIB_radiusAuthServTotalPacketsDropped 1,3,6,1,2,1,67,1,1,1,1,13
#define LEN_radiusAuthServTotalPacketsDropped 12
#define MIB_KEY_radiusAuthServTotalPacketsDropped 13
#define MIB_radiusAuthServTotalUnknownTypes 1,3,6,1,2,1,67,1,1,1,1,14
#define LEN_radiusAuthServTotalUnknownTypes 12
#define MIB_KEY_radiusAuthServTotalUnknownTypes 14
#define MIB_POS_radiusAuthClient 13
#define MIB_POS_radiusAuthClient_INDEX 14
#define MIB_radiusAuthClientTable 1,3,6,1,2,1,67,1,1,1,1,15
#define LEN_radiusAuthClientTable 12
#define MIB_KEY_radiusAuthClientTable 15
#define MIB_radiusAuthClientTable_X 1,3,6,1,2,1,67,1,1,1,1,15,0
#define LEN_radiusAuthClientTable_X 13
#define MIB_radiusAuthClientEntry 1,3,6,1,2,1,67,1,1,1,1,15,1
#define LEN_radiusAuthClientEntry 13
#define MIB_KEY_radiusAuthClientEntry 1
#define MIB_radiusAuthClientEntry_X 1,3,6,1,2,1,67,1,1,1,1,15,1,0
#define LEN_radiusAuthClientEntry_X 14
#define MIB_radiusAuthClientIndex 1,3,6,1,2,1,67,1,1,1,1,15,1,1
#define LEN_radiusAuthClientIndex 14
#define MIB_KEY_radiusAuthClientIndex 1
#define MIB_radiusAuthClientIndex_X 1,3,6,1,2,1,67,1,1,1,1,15,1,1,0
#define LEN_radiusAuthClientIndex_X 15
#define MIB_radiusAuthClientAddress 1,3,6,1,2,1,67,1,1,1,1,15,1,2
#define LEN_radiusAuthClientAddress 14
#define MIB_KEY_radiusAuthClientAddress 2
#define MIB_radiusAuthClientAddress_X 1,3,6,1,2,1,67,1,1,1,1,15,1,2,0
#define LEN_radiusAuthClientAddress_X 15
#define MIB_radiusAuthClientID 1,3,6,1,2,1,67,1,1,1,1,15,1,3
#define LEN_radiusAuthClientID 14
#define MIB_KEY_radiusAuthClientID 3
#define MIB_radiusAuthClientID_X 1,3,6,1,2,1,67,1,1,1,1,15,1,3,0
#define LEN_radiusAuthClientID_X 15
#define MIB_radiusAuthServAccessRequests 1,3,6,1,2,1,67,1,1,1,1,15,1,4
#define LEN_radiusAuthServAccessRequests 14
#define MIB_KEY_radiusAuthServAccessRequests 4
#define MIB_radiusAuthServAccessRequests_X 1,3,6,1,2,1,67,1,1,1,1,15,1,4,0
#define LEN_radiusAuthServAccessRequests_X 15
#define MIB_radiusAuthServDupAccessRequests 1,3,6,1,2,1,67,1,1,1,1,15,1,5
#define LEN_radiusAuthServDupAccessRequests 14
#define MIB_KEY_radiusAuthServDupAccessRequests 5
#define MIB_radiusAuthServDupAccessRequests_X 1,3,6,1,2,1,67,1,1,1,1,15,1,5,0
#define LEN_radiusAuthServDupAccessRequests_X 15
#define MIB_radiusAuthServAccessAccepts 1,3,6,1,2,1,67,1,1,1,1,15,1,6
#define LEN_radiusAuthServAccessAccepts 14
#define MIB_KEY_radiusAuthServAccessAccepts 6
#define MIB_radiusAuthServAccessAccepts_X 1,3,6,1,2,1,67,1,1,1,1,15,1,6,0
#define LEN_radiusAuthServAccessAccepts_X 15
#define MIB_radiusAuthServAccessRejects 1,3,6,1,2,1,67,1,1,1,1,15,1,7
#define LEN_radiusAuthServAccessRejects 14
#define MIB_KEY_radiusAuthServAccessRejects 7
#define MIB_radiusAuthServAccessRejects_X 1,3,6,1,2,1,67,1,1,1,1,15,1,7,0
#define LEN_radiusAuthServAccessRejects_X 15
#define MIB_radiusAuthServAccessChallenges 1,3,6,1,2,1,67,1,1,1,1,15,1,8
#define LEN_radiusAuthServAccessChallenges 14
#define MIB_KEY_radiusAuthServAccessChallenges 8
#define MIB_radiusAuthServAccessChallenges_X 1,3,6,1,2,1,67,1,1,1,1,15,1,8,0
#define LEN_radiusAuthServAccessChallenges_X 15
#define MIB_radiusAuthServMalformedAccessRequests 1,3,6,1,2,1,67,1,1,1,1,15,1,9
#define LEN_radiusAuthServMalformedAccessRequests 14
#define MIB_KEY_radiusAuthServMalformedAccessRequests 9
#define MIB_radiusAuthServMalformedAccessRequests_X 1,3,6,1,2,1,67,1,1,1,1,15,1,9,0
#define LEN_radiusAuthServMalformedAccessRequests_X 15
#define MIB_radiusAuthServBadAuthenticators 1,3,6,1,2,1,67,1,1,1,1,15,1,10
#define LEN_radiusAuthServBadAuthenticators 14
#define MIB_KEY_radiusAuthServBadAuthenticators 10
#define MIB_radiusAuthServBadAuthenticators_X 1,3,6,1,2,1,67,1,1,1,1,15,1,10,0
#define LEN_radiusAuthServBadAuthenticators_X 15
#define MIB_radiusAuthServPacketsDropped 1,3,6,1,2,1,67,1,1,1,1,15,1,11
#define LEN_radiusAuthServPacketsDropped 14
#define MIB_KEY_radiusAuthServPacketsDropped 11
#define MIB_radiusAuthServPacketsDropped_X 1,3,6,1,2,1,67,1,1,1,1,15,1,11,0
#define LEN_radiusAuthServPacketsDropped_X 15
#define MIB_radiusAuthServUnknownTypes 1,3,6,1,2,1,67,1,1,1,1,15,1,12
#define LEN_radiusAuthServUnknownTypes 14
#define MIB_KEY_radiusAuthServUnknownTypes 12
#define MIB_radiusAuthServUnknownTypes_X 1,3,6,1,2,1,67,1,1,1,1,15,1,12,0
#define LEN_radiusAuthServUnknownTypes_X 15
#define MIB_radiusAuthServMIBConformance 1,3,6,1,2,1,67,1,2
#define LEN_radiusAuthServMIBConformance 9
#define MIB_KEY_radiusAuthServMIBConformance 2
#define MIB_radiusAuthServMIBCompliances 1,3,6,1,2,1,67,1,2,1
#define LEN_radiusAuthServMIBCompliances 10
#define MIB_KEY_radiusAuthServMIBCompliances 1
#define MIB_radiusAuthServMIBCompliance 1,3,6,1,2,1,67,1,2,1,1
#define LEN_radiusAuthServMIBCompliance 11
#define MIB_KEY_radiusAuthServMIBCompliance 1
#define MIB_radiusAuthServMIBGroups 1,3,6,1,2,1,67,1,2,2
#define LEN_radiusAuthServMIBGroups 10
#define MIB_KEY_radiusAuthServMIBGroups 2
#define MIB_radiusAuthServMIBGroup 1,3,6,1,2,1,67,1,2,2,1
#define LEN_radiusAuthServMIBGroup 11
#define MIB_KEY_radiusAuthServMIBGroup 1
#define MIB_radiusAccounting 1,3,6,1,2,1,67,2
#define LEN_radiusAccounting 8
#define MIB_KEY_radiusAccounting 2
#define MIB_radiusAccServMIB 1,3,6,1,2,1,67,2,1
#define LEN_radiusAccServMIB 9
#define MIB_KEY_radiusAccServMIB 1
#define MIB_radiusAccServMIBObjects 1,3,6,1,2,1,67,2,1,1
#define LEN_radiusAccServMIBObjects 10
#define MIB_KEY_radiusAccServMIBObjects 1
#define MIB_radiusAccServ 1,3,6,1,2,1,67,2,1,1,1
#define LEN_radiusAccServ 11
#define MIB_KEY_radiusAccServ 1
#define MIB_POS_radiusAccServ 11
#define MIB_radiusAccServIdent 1,3,6,1,2,1,67,2,1,1,1,1
#define LEN_radiusAccServIdent 12
#define MIB_KEY_radiusAccServIdent 1
#define MIB_radiusAccServUpTime 1,3,6,1,2,1,67,2,1,1,1,2
#define LEN_radiusAccServUpTime 12
#define MIB_KEY_radiusAccServUpTime 2
#define MIB_radiusAccServResetTime 1,3,6,1,2,1,67,2,1,1,1,3
#define LEN_radiusAccServResetTime 12
#define MIB_KEY_radiusAccServResetTime 3
#define MIB_radiusAccServConfigReset 1,3,6,1,2,1,67,2,1,1,1,4
#define LEN_radiusAccServConfigReset 12
#define MIB_KEY_radiusAccServConfigReset 4
#define MIB_radiusAccServTotalRequests 1,3,6,1,2,1,67,2,1,1,1,5
#define LEN_radiusAccServTotalRequests 12
#define MIB_KEY_radiusAccServTotalRequests 5
#define MIB_radiusAccServTotalInvalidRequests 1,3,6,1,2,1,67,2,1,1,1,6
#define LEN_radiusAccServTotalInvalidRequests 12
#define MIB_KEY_radiusAccServTotalInvalidRequests 6
#define MIB_radiusAccServTotalDupRequests 1,3,6,1,2,1,67,2,1,1,1,7
#define LEN_radiusAccServTotalDupRequests 12
#define MIB_KEY_radiusAccServTotalDupRequests 7
#define MIB_radiusAccServTotalResponses 1,3,6,1,2,1,67,2,1,1,1,8
#define LEN_radiusAccServTotalResponses 12
#define MIB_KEY_radiusAccServTotalResponses 8
#define MIB_radiusAccServTotalMalformedRequests 1,3,6,1,2,1,67,2,1,1,1,9
#define LEN_radiusAccServTotalMalformedRequests 12
#define MIB_KEY_radiusAccServTotalMalformedRequests 9
#define MIB_radiusAccServTotalBadAuthenticators 1,3,6,1,2,1,67,2,1,1,1,10
#define LEN_radiusAccServTotalBadAuthenticators 12
#define MIB_KEY_radiusAccServTotalBadAuthenticators 10
#define MIB_radiusAccServTotalPacketsDropped 1,3,6,1,2,1,67,2,1,1,1,11
#define LEN_radiusAccServTotalPacketsDropped 12
#define MIB_KEY_radiusAccServTotalPacketsDropped 11
#define MIB_radiusAccServTotalNoRecords 1,3,6,1,2,1,67,2,1,1,1,12
#define LEN_radiusAccServTotalNoRecords 12
#define MIB_KEY_radiusAccServTotalNoRecords 12
#define MIB_radiusAccServTotalUnknownTypes 1,3,6,1,2,1,67,2,1,1,1,13
#define LEN_radiusAccServTotalUnknownTypes 12
#define MIB_KEY_radiusAccServTotalUnknownTypes 13
#define MIB_radiusAccClientTable 1,3,6,1,2,1,67,2,1,1,1,14
#define LEN_radiusAccClientTable 12
#define MIB_KEY_radiusAccClientTable 14
#define MIB_POS_radiusAccClient 13
#define MIB_POS_radiusAccClient_INDEX 14
#define MIB_radiusAccClientEntry 1,3,6,1,2,1,67,2,1,1,1,14,1
#define LEN_radiusAccClientEntry 13
#define MIB_KEY_radiusAccClientEntry 1
#define MIB_radiusAccClientEntry_X 1,3,6,1,2,1,67,2,1,1,1,14,1,0
#define LEN_radiusAccClientEntry_X 14
#define MIB_radiusAccClientIndex 1,3,6,1,2,1,67,2,1,1,1,14,1,1
#define LEN_radiusAccClientIndex 14
#define MIB_KEY_radiusAccClientIndex 1
#define MIB_radiusAccClientIndex_X 1,3,6,1,2,1,67,2,1,1,1,14,1,1,0
#define LEN_radiusAccClientIndex_X 15
#define MIB_radiusAccClientAddress 1,3,6,1,2,1,67,2,1,1,1,14,1,2
#define LEN_radiusAccClientAddress 14
#define MIB_KEY_radiusAccClientAddress 2
#define MIB_radiusAccClientAddress_X 1,3,6,1,2,1,67,2,1,1,1,14,1,2,0
#define LEN_radiusAccClientAddress_X 15
#define MIB_radiusAccClientID 1,3,6,1,2,1,67,2,1,1,1,14,1,3
#define LEN_radiusAccClientID 14
#define MIB_KEY_radiusAccClientID 3
#define MIB_radiusAccClientID_X 1,3,6,1,2,1,67,2,1,1,1,14,1,3,0
#define LEN_radiusAccClientID_X 15
#define MIB_radiusAccServPacketsDropped 1,3,6,1,2,1,67,2,1,1,1,14,1,4
#define LEN_radiusAccServPacketsDropped 14
#define MIB_KEY_radiusAccServPacketsDropped 4
#define MIB_radiusAccServPacketsDropped_X 1,3,6,1,2,1,67,2,1,1,1,14,1,4,0
#define LEN_radiusAccServPacketsDropped_X 15
#define MIB_radiusAccServRequests 1,3,6,1,2,1,67,2,1,1,1,14,1,5
#define LEN_radiusAccServRequests 14
#define MIB_KEY_radiusAccServRequests 5
#define MIB_radiusAccServRequests_X 1,3,6,1,2,1,67,2,1,1,1,14,1,5,0
#define LEN_radiusAccServRequests_X 15
#define MIB_radiusAccServDupRequests 1,3,6,1,2,1,67,2,1,1,1,14,1,6
#define LEN_radiusAccServDupRequests 14
#define MIB_KEY_radiusAccServDupRequests 6
#define MIB_radiusAccServDupRequests_X 1,3,6,1,2,1,67,2,1,1,1,14,1,6,0
#define LEN_radiusAccServDupRequests_X 15
#define MIB_radiusAccServResponses 1,3,6,1,2,1,67,2,1,1,1,14,1,7
#define LEN_radiusAccServResponses 14
#define MIB_KEY_radiusAccServResponses 7
#define MIB_radiusAccServResponses_X 1,3,6,1,2,1,67,2,1,1,1,14,1,7,0
#define LEN_radiusAccServResponses_X 15
#define MIB_radiusAccServBadAuthenticators 1,3,6,1,2,1,67,2,1,1,1,14,1,8
#define LEN_radiusAccServBadAuthenticators 14
#define MIB_KEY_radiusAccServBadAuthenticators 8
#define MIB_radiusAccServBadAuthenticators_X 1,3,6,1,2,1,67,2,1,1,1,14,1,8,0
#define LEN_radiusAccServBadAuthenticators_X 15
#define MIB_radiusAccServMalformedRequests 1,3,6,1,2,1,67,2,1,1,1,14,1,9
#define LEN_radiusAccServMalformedRequests 14
#define MIB_KEY_radiusAccServMalformedRequests 9
#define MIB_radiusAccServMalformedRequests_X 1,3,6,1,2,1,67,2,1,1,1,14,1,9,0
#define LEN_radiusAccServMalformedRequests_X 15
#define MIB_radiusAccServNoRecords 1,3,6,1,2,1,67,2,1,1,1,14,1,10
#define LEN_radiusAccServNoRecords 14
#define MIB_KEY_radiusAccServNoRecords 10
#define MIB_radiusAccServNoRecords_X 1,3,6,1,2,1,67,2,1,1,1,14,1,10,0
#define LEN_radiusAccServNoRecords_X 15
#define MIB_radiusAccServUnknownTypes 1,3,6,1,2,1,67,2,1,1,1,14,1,11
#define LEN_radiusAccServUnknownTypes 14
#define MIB_KEY_radiusAccServUnknownTypes 11
#define MIB_radiusAccServUnknownTypes_X 1,3,6,1,2,1,67,2,1,1,1,14,1,11,0
#define LEN_radiusAccServUnknownTypes_X 15
#define MIB_radiusAccServMIBConformance 1,3,6,1,2,1,67,2,1,2
#define LEN_radiusAccServMIBConformance 10
#define MIB_KEY_radiusAccServMIBConformance 2
#define MIB_radiusAccServMIBCompliances 1,3,6,1,2,1,67,2,1,2,1
#define LEN_radiusAccServMIBCompliances 11
#define MIB_KEY_radiusAccServMIBCompliances 1
#define MIB_radiusAccServMIBCompliance 1,3,6,1,2,1,67,2,1,2,1,1
#define LEN_radiusAccServMIBCompliance 12
#define MIB_KEY_radiusAccServMIBCompliance 1
#define MIB_radiusAccServMIBGroups 1,3,6,1,2,1,67,2,1,2,2
#define LEN_radiusAccServMIBGroups 11
#define MIB_KEY_radiusAccServMIBGroups 2
#define MIB_radiusAccServMIBGroup 1,3,6,1,2,1,67,2,1,2,2,1
#define LEN_radiusAccServMIBGroup 12
#define MIB_KEY_radiusAccServMIBGroup 1
#define MIB_radiusStatistics 1,3,6,1,2,1,67,3
#define LEN_radiusStatistics 8
#define MIB_KEY_radiusStatistics 3
#define MIB_radiusStatMIB 1,3,6,1,2,1,67,3,1
#define LEN_radiusStatMIB 9
#define MIB_KEY_radiusStatMIB 1
#define MIB_POS_radiusStatServ 9
#define MIB_radiusStatIdent 1,3,6,1,2,1,67,3,1,1
#define LEN_radiusStatIdent 10
#define MIB_KEY_radiusStatIdent 1
#define MIB_radiusStatUpTime 1,3,6,1,2,1,67,3,1,2
#define LEN_radiusStatUpTime 10
#define MIB_KEY_radiusStatUpTime 2
#define MIB_radiusStatConfigReset 1,3,6,1,2,1,67,3,1,3
#define LEN_radiusStatConfigReset 10
#define MIB_KEY_radiusStatConfigReset 3
#define MIB_radiusStatTotalLines 1,3,6,1,2,1,67,3,1,4
#define LEN_radiusStatTotalLines 10
#define MIB_KEY_radiusStatTotalLines 4
#define MIB_radiusStatTotalLinesInUse 1,3,6,1,2,1,67,3,1,5
#define LEN_radiusStatTotalLinesInUse 10
#define MIB_KEY_radiusStatTotalLinesInUse 5
#define MIB_radiusStatTotalLinesIdle 1,3,6,1,2,1,67,3,1,6
#define LEN_radiusStatTotalLinesIdle 10
#define MIB_KEY_radiusStatTotalLinesIdle 6
#define MIB_radiusStatNASTable 1,3,6,1,2,1,67,3,1,7
#define LEN_radiusStatNASTable 10
#define MIB_KEY_radiusStatNASTable 7
#define MIB_radiusStatNASEntry 1,3,6,1,2,1,67,3,1,7,1
#define LEN_radiusStatNASEntry 11
#define MIB_KEY_radiusStatNASEntry 1
#define MIB_POS_radiusStatNASEntry 11
#define MIB_POS_radiusStatNASEntry_INDEX 12
#define MIB_NASIndex 1,3,6,1,2,1,67,3,1,7,1,1
#define LEN_NASIndex 12
#define MIB_KEY_NASIndex 1
#define MIB_NASIndex_X 1,3,6,1,2,1,67,3,1,7,1,1,0
#define LEN_NASIndex_X 13
#define MIB_NASAddress 1,3,6,1,2,1,67,3,1,7,1,2
#define LEN_NASAddress 12
#define MIB_KEY_NASAddress 2
#define MIB_NASAddress_X 1,3,6,1,2,1,67,3,1,7,1,2,0
#define LEN_NASAddress_X 13
#define MIB_NASID 1,3,6,1,2,1,67,3,1,7,1,3
#define LEN_NASID 12
#define MIB_KEY_NASID 3
#define MIB_NASID_X 1,3,6,1,2,1,67,3,1,7,1,3,0
#define LEN_NASID_X 13
#define MIB_NASLines 1,3,6,1,2,1,67,3,1,7,1,4
#define LEN_NASLines 12
#define MIB_KEY_NASLines 4
#define MIB_NASLines_X 1,3,6,1,2,1,67,3,1,7,1,4,0
#define LEN_NASLines_X 13
#define MIB_NASLinesInUse 1,3,6,1,2,1,67,3,1,7,1,5
#define LEN_NASLinesInUse 12
#define MIB_KEY_NASLinesInUse 5
#define MIB_NASLinesInUse_X 1,3,6,1,2,1,67,3,1,7,1,5,0
#define LEN_NASLinesInUse_X 13
#define MIB_NASLinesIdle 1,3,6,1,2,1,67,3,1,7,1,6
#define LEN_NASLinesIdle 12
#define MIB_KEY_NASLinesIdle 6
#define MIB_NASLinesIdle_X 1,3,6,1,2,1,67,3,1,7,1,6,0
#define LEN_NASLinesIdle_X 13
#define MIB_radiusStatNASPortTable 1,3,6,1,2,1,67,3,1,8
#define LEN_radiusStatNASPortTable 10
#define MIB_KEY_radiusStatNASPortTable 8
#define MIB_radiusStatNASPortTable_X 1,3,6,1,2,1,67,3,1,8,0
#define LEN_radiusStatNASPortTable_X 11
#define MIB_radiusStatNASPortEntry 1,3,6,1,2,1,67,3,1,8,1
#define LEN_radiusStatNASPortEntry 11
#define MIB_KEY_radiusStatNASPortEntry 1
#define MIB_radiusStatNASPortEntry_X 1,3,6,1,2,1,67,3,1,8,1,0
#define LEN_radiusStatNASPortEntry_X 12
#define MIB_POS_NASPortEntry 11
#define MIB_POS_NASPortEntry_INDEX 12
#define MIB_radiusStatPortIndex 1,3,6,1,2,1,67,3,1,8,1,1
#define LEN_radiusStatPortIndex 12
#define MIB_KEY_radiusStatPortIndex 1
#define MIB_radiusStatPortIndex_X 1,3,6,1,2,1,67,3,1,8,1,1,0
#define LEN_radiusStatPortIndex_X 13
#define MIB_radiusStatNASIndex 1,3,6,1,2,1,67,3,1,8,1,2
#define LEN_radiusStatNASIndex 12
#define MIB_KEY_radiusStatNASIndex 2
#define MIB_radiusStatNASIndex_X 1,3,6,1,2,1,67,3,1,8,1,2,0
#define LEN_radiusStatNASIndex_X 13
#define MIB_radiusStatPortID 1,3,6,1,2,1,67,3,1,8,1,3
#define LEN_radiusStatPortID 12
#define MIB_KEY_radiusStatPortID 3
#define MIB_radiusStatPortID_X 1,3,6,1,2,1,67,3,1,8,1,3,0
#define LEN_radiusStatPortID_X 13
#define MIB_radiusStatPortFramedAddress 1,3,6,1,2,1,67,3,1,8,1,4
#define LEN_radiusStatPortFramedAddress 12
#define MIB_KEY_radiusStatPortFramedAddress 4
#define MIB_radiusStatPortFramedAddress_X 1,3,6,1,2,1,67,3,1,8,1,4,0
#define LEN_radiusStatPortFramedAddress_X 13
#define MIB_radiusStatPortTotalLogins 1,3,6,1,2,1,67,3,1,8,1,5
#define LEN_radiusStatPortTotalLogins 12
#define MIB_KEY_radiusStatPortTotalLogins 5
#define MIB_radiusStatPortTotalLogins_X 1,3,6,1,2,1,67,3,1,8,1,5,0
#define LEN_radiusStatPortTotalLogins_X 13
#define MIB_radiusStatPortStatus 1,3,6,1,2,1,67,3,1,8,1,6
#define LEN_radiusStatPortStatus 12
#define MIB_KEY_radiusStatPortStatus 6
#define MIB_radiusStatPortStatus_X 1,3,6,1,2,1,67,3,1,8,1,6,0
#define LEN_radiusStatPortStatus_X 13
#define MIB_radiusStatPortStatusDate 1,3,6,1,2,1,67,3,1,8,1,7
#define LEN_radiusStatPortStatusDate 12
#define MIB_KEY_radiusStatPortStatusDate 7
#define MIB_radiusStatPortStatusDate_X 1,3,6,1,2,1,67,3,1,8,1,7,0
#define LEN_radiusStatPortStatusDate_X 13
#define MIB_radiusStatPortUpTime 1,3,6,1,2,1,67,3,1,8,1,8
#define LEN_radiusStatPortUpTime 12
#define MIB_KEY_radiusStatPortUpTime 8
#define MIB_radiusStatPortUpTime_X 1,3,6,1,2,1,67,3,1,8,1,8,0
#define LEN_radiusStatPortUpTime_X 13
#define MIB_radiusStatPortLastLoginName 1,3,6,1,2,1,67,3,1,8,1,9
#define LEN_radiusStatPortLastLoginName 12
#define MIB_KEY_radiusStatPortLastLoginName 9
#define MIB_radiusStatPortLastLoginName_X 1,3,6,1,2,1,67,3,1,8,1,9,0
#define LEN_radiusStatPortLastLoginName_X 13
#define MIB_radiusStatPortLastLoginDate 1,3,6,1,2,1,67,3,1,8,1,10
#define LEN_radiusStatPortLastLoginDate 12
#define MIB_KEY_radiusStatPortLastLoginDate 10
#define MIB_radiusStatPortLastLoginDate_X 1,3,6,1,2,1,67,3,1,8,1,10,0
#define LEN_radiusStatPortLastLoginDate_X 13
#define MIB_radiusStatPortLastLogoutDate 1,3,6,1,2,1,67,3,1,8,1,11
#define LEN_radiusStatPortLastLogoutDate 12
#define MIB_KEY_radiusStatPortLastLogoutDate 11
#define MIB_radiusStatPortLastLogoutDate_X 1,3,6,1,2,1,67,3,1,8,1,11,0
#define LEN_radiusStatPortLastLogoutDate_X 13
#define MIB_radiusStatPortIdleTotalTime 1,3,6,1,2,1,67,3,1,8,1,12
#define LEN_radiusStatPortIdleTotalTime 12
#define MIB_KEY_radiusStatPortIdleTotalTime 12
#define MIB_radiusStatPortIdleTotalTime_X 1,3,6,1,2,1,67,3,1,8,1,12,0
#define LEN_radiusStatPortIdleTotalTime_X 13
#define MIB_radiusStatPortIdleMaxTime 1,3,6,1,2,1,67,3,1,8,1,13
#define LEN_radiusStatPortIdleMaxTime 12
#define MIB_KEY_radiusStatPortIdleMaxTime 13
#define MIB_radiusStatPortIdleMaxTime_X 1,3,6,1,2,1,67,3,1,8,1,13,0
#define LEN_radiusStatPortIdleMaxTime_X 13
#define MIB_radiusStatPortIdleMaxDate 1,3,6,1,2,1,67,3,1,8,1,14
#define LEN_radiusStatPortIdleMaxDate 12
#define MIB_KEY_radiusStatPortIdleMaxDate 14
#define MIB_radiusStatPortIdleMaxDate_X 1,3,6,1,2,1,67,3,1,8,1,14,0
#define LEN_radiusStatPortIdleMaxDate_X 13
#define MIB_radiusStatPortInUseTotalTime 1,3,6,1,2,1,67,3,1,8,1,15
#define LEN_radiusStatPortInUseTotalTime 12
#define MIB_KEY_radiusStatPortInUseTotalTime 15
#define MIB_radiusStatPortInUseTotalTime_X 1,3,6,1,2,1,67,3,1,8,1,15,0
#define LEN_radiusStatPortInUseTotalTime_X 13
#define MIB_radiusStatPortInUseMaxTime 1,3,6,1,2,1,67,3,1,8,1,16
#define LEN_radiusStatPortInUseMaxTime 12
#define MIB_KEY_radiusStatPortInUseMaxTime 16
#define MIB_radiusStatPortInUseMaxTime_X 1,3,6,1,2,1,67,3,1,8,1,16,0
#define LEN_radiusStatPortInUseMaxTime_X 13
#define MIB_radiusStatPortInUseMaxDate 1,3,6,1,2,1,67,3,1,8,1,17
#define LEN_radiusStatPortInUseMaxDate 12
#define MIB_KEY_radiusStatPortInUseMaxDate 17
#define MIB_radiusStatPortInUseMaxDate_X 1,3,6,1,2,1,67,3,1,8,1,17,0
#define LEN_radiusStatPortInUseMaxDate_X 13
