/* This file is part of GNU RADIUS.
 * Copyright (C) 2000, Sergey Poznyakoff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
/* $Id: radwho.h,v 1.3 2000/12/18 17:55:11 gray Exp $ */

typedef struct format Format;
struct format {
	Format *next;
	char *name;
	char *hdr;
	char *(*fmt)();
	int offset;
	int width;
	int prec;
};

extern char *eol;
extern char *timefmt;

void fmtout(FILE *fp, void *data, Format *ctl);
void fmtheader(FILE *fp, Format *ctl);
Format * make_format(char *str);

